@@ -4,7 +4,6 @@
 from __future__ import unicode_literals
 from __future__ import absolute_import
 
-import io
 import os
 import stat
 import socket
@@ -12,14 +11,13 @@ import sys
 
 import six
 import paramiko
-from cached_property import threaded_cached_property as cached_property
+from property_cached import threaded_cached_property as cached_property
 
 from .. import errors
 from ..base import FS
 from ..info import Info
 from ..enums import ResourceType
-from ..iotools import RawWrapper
-from ..path import basename
+from ..path import basename, dirname
 from ..permissions import Permissions
 from ..osfs import OSFS
 from ..mode import Mode
@@ -52,12 +50,12 @@ class SSHFS(FS):
             ``CreateFailed`` error.
     """
 
-
     _meta = {
         'case_insensitive': False,
         'invalid_path_chars': '\0',
         'network': True,
         'read_only': False,
+        'supports_rename': True,
         'thread_safe': sys.version_info[:2] > (3, 4),
         'unicode_paths': True,
         'virtual': False,
@@ -88,7 +86,7 @@ class SSHFS(FS):
         config = ssh_config.lookup(host)
         pkey = config.get('identityfile') or pkey
         # Extract the given info
-        pkey, keyfile= (pkey, None) \
+        pkey, keyfile = (pkey, None) \
             if isinstance(pkey, paramiko.PKey) else (None, pkey)
         self._user = user = user or config.get('user')
         self._host = host = config.get('hostname')
@@ -120,7 +118,7 @@ class SSHFS(FS):
             self._sftp = client.open_sftp()
 
         except (paramiko.ssh_exception.SSHException,            # protocol errors
-                paramiko.ssh_exception.NoValidConnectionsError, # connexion errors
+                paramiko.ssh_exception.NoValidConnectionsError,  # connexion errors
                 socket.gaierror, socket.timeout) as e:          # TCP errors
 
             message = "Unable to create filesystem: {}".format(e)
@@ -174,14 +172,38 @@ class SSHFS(FS):
 
         return self.opendir(path)
 
+    def move(self, src_path, dst_path, overwrite=False):
+        self.check()
+        _src_path = self.validatepath(src_path)
+        _dst_path = self.validatepath(dst_path)
+
+        with self._lock:
+            # check src exists and is a file
+            src_info = self.getinfo(_src_path)
+            if src_info.is_dir:
+                raise errors.FileExpected(src_path)
+            # check dst is not a dir and can be created
+            if self.isdir(_dst_path):
+                raise errors.FileExpected(dst_path)
+            if not self.isdir(dirname(_dst_path)):
+                raise errors.ResourceNotFound(dirname(dst_path))
+            # check dst does not exist or remove it
+            if self.isfile(_dst_path):
+                if not overwrite:
+                    raise errors.DestinationExists(dst_path)
+                with convert_sshfs_errors('remove', dst_path):
+                    self._sftp.remove(_dst_path)
+            # rename the file through SFTP's 'RENAME'
+            self._sftp.rename(_src_path, _dst_path)
+
     def openbin(self, path, mode='r', buffering=-1, **options):  # noqa: D102
         """Open a binary file-like object.
 
         Arguments:
             path (str): A path on the filesystem.
             mode (str): Mode to open the file (must be a valid, non-text mode).
-                Since this method only opens binary files, the ``b`` in the mode
-                is implied.
+                Since this method only opens binary files, the ``b`` in the
+                mode is implied.
             buffering (int): the buffering policy (-1 to use default buffering,
                 0 to disable completely, 1 to enable line based buffering, or
                 any larger positive integer for a custom buffer size).
@@ -189,6 +211,9 @@ class SSHFS(FS):
         Keyword Arguments:
             pipelined (bool): Set the transfer in pipelined mode (should
                 improve transfer speed). Defaults to ``True``.
+            prefetch (bool): Use background threading to prefetch the file
+                content when opened in reading mode. Disable in case of
+                threading issues. Defaults to ``True``.
 
         Raises:
             fs.errors.FileExpected: if the path if not a file.
@@ -210,7 +235,7 @@ class SSHFS(FS):
                     raise errors.FileExists(path)
                 else:
                     _mode = Mode(''.join(set(mode.replace('x', 'w'))))
-            elif _mode.reading and not _mode.create and not self.exists(_path):
+            elif not _mode.create and not self.exists(_path):
                 raise errors.ResourceNotFound(path)
             elif self.isdir(_path):
                 raise errors.FileExpected(path)
@@ -222,6 +247,9 @@ class SSHFS(FS):
                     bufsize=buffering
                 )
                 handle.set_pipelined(options.get("pipelined", True))
+                if options.get("prefetch", True):
+                    if _mode.reading and not _mode.writing:
+                        handle.prefetch(self.getsize(_path))
                 return SSHFile(handle)
 
     def remove(self, path):  # noqa: D102
@@ -244,12 +272,12 @@ class SSHFS(FS):
         # NB: this will raise ResourceNotFound
         # and DirectoryExpected as expected by
         # the specifications
-        if not self.isempty(path):
+        if not self.isempty(_path):
             raise errors.DirectoryNotEmpty(path)
 
         with convert_sshfs_errors('removedir', path):
             with self._lock:
-                self._sftp.rmdir(path)
+                self._sftp.rmdir(_path)
 
     def setinfo(self, path, info):  # noqa: D102
         self.check()
@@ -263,15 +291,15 @@ class SSHFS(FS):
 
         with convert_sshfs_errors('setinfo', path):
             if 'accessed' in details or 'modified' in details:
-                self._utime(path,
+                self._utime(_path,
                             details.get("modified"),
                             details.get("accessed"))
             if 'uid' in access or 'gid' in access:
-                self._chown(path,
+                self._chown(_path,
                             access.get('uid'),
                             access.get('gid'))
             if 'permissions' in access:
-                self._chmod(path, access['permissions'].mode)
+                self._chmod(_path, access['permissions'].mode)
 
     @cached_property
     def platform(self):
@@ -280,8 +308,11 @@ class SSHFS(FS):
         Returns:
             str: the platform of the remote server, as in `sys.platform`.
         """
-        uname_sys = self._exec_command("uname -s")
-        sysinfo = self._exec_command("sysinfo")
+        try:
+            uname_sys = self._exec_command("uname -s")
+            sysinfo = self._exec_command("sysinfo")
+        except paramiko.ssh_exception.SSHException:
+            return "unknown"
         if uname_sys is not None:
             if uname_sys == b"FreeBSD":
                 return "freebsd"
@@ -350,7 +381,7 @@ class SSHFS(FS):
         }
 
         details['created'] = getattr(stat_result, 'st_birthtime', None)
-        ctime_key = 'created' if self.platform=="win32" else 'metadata_changed'
+        ctime_key = 'created' if self.platform == "win32" else 'metadata_changed'
         details[ctime_key] = getattr(stat_result, 'st_ctime', None)
         return details
 
